﻿if not myAutoMacro then
	return
end

-- library initialization
local MAM = myAutoMacro
local L = LibStub("AceLocale-3.0"):GetLocale("myAutoMacro", true)
local ACR = LibStub("AceConfigRegistry-3.0")
local AIS = LibStub("LibAdvancedIconSelector-1.0", true)

-- cache
local print, string, tonumber, tostring, format, wipe, table, type, pairs, ipairs  = print, string, tonumber, tostring, format, wipe, table, type, pairs, ipairs
local _

-- constants
MAM.ICON_MAX_BUTTON = 50
MAM.NUMBER_ICON_PER_RAW = 10
MAM.ICON_RAW_HEIGHT = 35
MAM.MACRO_ICON_PATH = "Interface\\Icons\\"



-- housekeeping functions ------------------

local function DeepCopy(src)
	local res = {}
    for k, v in pairs(src) do
        if type(v) == "table" then
            v = DeepCopy(v)
        end
        res[k] = v
    end
    return res
end

local function AddCopy(from, recipient)
    for k, v in pairs(from) do
        if type(v) == "table" then
            v = DeepCopy(v)
        end
        recipient[k] = v
    end
end

local function HlString(str)
	return MAM.HIGHLIGHT_COLOR .. str .. MAM.NORMAL_COLOR
end



-- Table options ----------------------------

-- dummy options
function MAM:DummyOptions()
	local options = {
		type = "group",
		name = "myAutoMacro",
		childGroups = "tab",
		disabled = function()
			return MAM.inCombat
		end,
		args = {
			toggleOptions = {
				type = "execute",
				order = 1,
				name = L["myAutoMacro config"],
				desc = L["Toggles myAutoMacro configuration panel."],
				func = function()
					MAM:LoadOptions()
				end,
			},
		},
	}

	return options
end

-- real options
function MAM:Options()
	local args = {
		associations = {
			type = "execute",
			order = 1,
			name = L["Show associations"],
			desc = L["Click to show all current associations of automacros."],
			func = function()
				MAM:ShowAssociations()
			end,
		},		
		general = {
			type = "group",
			order = 5,
			name = L["General"],
			desc = L["General settings."],
			get = function(info) return MAM.db.profile.general[info[#info]] end,
			set = function(info, value)
				MAM.db.profile.general[info[#info]] = value
			end,
			args = MAM:GeneralOptions(),
		},
		automacros = {
			type = "group",
			order = 10,
			childGroups = "select",
			name = "Automacros",
			desc = L["Associate macros form the library to in game automacros here."],
			args = MAM:AutomacrosOptions(),
		},
		library = {
			type = "group",
			order = 15,
			name = L["Macro library"],
			desc = L["Create, delete and manage your macros here."],
			args = MAM:LibraryOptions(),
		},
		communication = {
			type = "group",
			order = 20,
			name = L["Broadcast"],
			desc = L["Tools to help you share macros with other players."],
			args = MAM:ComOptions(),
		},
		npc = {
			type = "group",
			order = 25,
			name = L["Custom NPC"],
			desc = L["Manage custom NPC here."],
			args = MAM:NPCOptions(),
		},
		zone = {
			type = "group",
			order = 30,
			name = L["Custom Zones"],
			desc = L["Manage custom zones here."],
			args = MAM:ZoneOptions(),
		},
	}
			
	return args
end

-- general tab
function MAM:GeneralOptions()
	local currentMacro = 0
	local currentAutomacro = 1
	
	local args = {
		macroNumber = {
			type = "range",
			order = 3,
			name = L["Number of automacro"],
			desc = L["This is the number of in game macro automatically created and updated by myAutoMacro."],
			min = 1, max = 5, step = 1,
			set = function(info, value)
				MAM.db.profile.general.macroNumber = value
				MAM:CreateMacros()
				currentAutomacro = 1
			end,
		},
		space1 = {
			type = "description",
			order = 5,
			name = " ",
			width = "full",
		},
		verbose = {
			type = "toggle",
			order = 11,
			name = L["Verbose"],
			desc = L["Prints messages in chat when macros are swapped."],
		},
		splashMessage = {
			type = "toggle",
			order = 13,
			name = L["Welcome message"],
		},
		space2 = {
			type = "description",
			order = 20,
			name = " ",
			width = "full",
		},
		space21 = {
			type = "description",
			order = 21,
			name = " ",
			width = "full",
		},
		descForceSwitch = {
			type = "description",
			order = 30,
			name = MAM.NORMAL_COLOR .. L["Use this to switch immediatly the given automacro to the selected macro:"],
			fontSize = "medium",
			width = "full",
		},
		selectMacro = {
			type = "select",
			order = 35,
			name = L["Select a macro"],
			values = function()
				local list = {}
				for i = 1, #MAM.db.global.library do
					list[i] = MAM.db.global.library[i].name
				end
				list[0] = L["None"]

				return list
			end,
			get = function()
				return currentMacro
			end,
			set = function(info, value)
				currentMacro = value
			end,
		},
		selectAutomacro = {
			type = "select",
			order = 40,
			disabled = function()
				return currentMacro == 0
			end,
			name = L["Select an automacro"],
			values = function()
				local list = {}
				for i = 1, MAM.db.profile.general.macroNumber do
					list[i] = "Automacro" .. i
				end
				return list
			end,
			get = function()
				return currentAutomacro
			end,
			set = function(info, value)
				currentAutomacro = value
			end,
		},
		forceSwitchExec = {
			type = "execute",
			order = 50,
			disabled = function()
				return currentMacro == 0
			end,
			name = L["Force switch"],
			desc = L["Will switch immediatly this automacro to the above macro."],
			func = function()
				MAM:SwitchMacro(currentMacro, currentAutomacro)
			end,
		},
	}
		
	return args
end

-- library tab
function MAM:LibraryOptions()
	local currentMacro = 0
	
	-- limits the length of addon macro name
	local function ValidateInput(info, value)
		local isValid = false
		if value and string.len(value) <= 20 and value ~= L["New"] then
			isValid = true
		else
			print(MAM.FORMATED_ADDON_NAME .. L["This name is not valid. Please do not enter more than 20 characters."])
		end
		
		return isValid
	end
	
	-- limits the size of macro body to 255
	local function MacroBodySizeValidate(info, value)
		local isValid = false
		if value and string.len(value) <= 255 then
			isValid = true
		else
			print(MAM.FORMATED_ADDON_NAME .. L["This macro is too long. Please do not exceed 255 characters."])
		end
		
		return isValid
	end
	
	-- displays the character number in macro body
	local function MacroBodySize()
		local number = 0
		if currentMacro ~= 0 then
			number = string.len(MAM.db.global.library[currentMacro].body)
		end
		
		return number
	end
	
	-- options
	local args = {
		select = {
			type = "select",
			order = 0,
			name = L["Select a macro"],
			desc = L["Select \"New\" to create a new macro."],
			values = function()
				local list = {}
				for i = 1, #MAM.db.global.library do
					list[i] = MAM.db.global.library[i].name
				end
				list[0] = L["New"]

				return list
			end,
			get = function()
				return currentMacro
			end,
			set = function(info, value)
				currentMacro = value
			end,
		},
		delete = {
			type = "execute",
			order = 2,
			name = L["Delete"],
			desc = L["Deletes the selected macro."],
			disabled = function()
				return currentMacro == 0
			end,
			confirm = true,
			confirmText = L["This will delete the macro and will reset every association between this macro and in game automacros. Do you want to continue?"],
			func = function()				
				-- clears ig macro and current macro
				for i = 1, MAM.db.profile.general.macroNumber do
					if MAM.db.profile.currentMacros[i] == currentMacro then
						EditMacro(MAM.MACRO_PREFIX .. i .. MAM.MACRO_SUFFIX .. MAM.db.global.library[MAM.db.profile.currentMacros[i]].name, MAM.MACRO_PREFIX .. i .. MAM.MACRO_SUFFIX, MAM.MACRO_DEFAULT_ICON, "")
						MAM.db.profile.currentMacros[i] = ""
					end
				end
				
				-- clears library
				wipe(MAM.db.global.library[currentMacro])
				MAM.db.global.library[currentMacro] = "deleted"
				
				-- clears any association
				MAM:MonsieurPropre("macro")
				
				currentMacro = 0
			end,
		},
		space1 = {
			type = "description",
			order = 3,
			name = " ",
			width = "full",
		},
		name = {
			type = "input",
			order = 4,
			name = L["Macro name"],
			validate = ValidateInput,
			get = function()
				if currentMacro ~= 0 then
					return MAM.db.global.library[currentMacro].name
				end
			end,
			set = function(info, value)
				local libID = currentMacro
				if libID == 0 then
					libID = #MAM.db.global.library + 1
					MAM.db.global.library[libID] = {}
					MAM.db.global.library[libID].body = ""
					MAM.db.global.library[libID].texture = MAM.MACRO_DEFAULT_ICON
				end
				local oldName = MAM.db.global.library[libID].name
				MAM.db.global.library[libID].name = value
				currentMacro = libID
				for i = 1, MAM.db.profile.general.macroNumber do
					if MAM.db.profile.currentMacros[i] == currentMacro then
						MAM:UpdateIgMacro(i, oldName, "name", MAM.db.global.library[libID].name)
					end
				end
			end,
		},
		icon = {
			type = "execute",
			order = 7,
			name = MAM.NORMAL_COLOR .. L["Macro icon"],
			desc = L["Click to open icon selection pannel."],
			disabled = function()
				return currentMacro == 0
			end,
			image = function()
				if currentMacro ~= 0 then
					return MAM.MACRO_ICON_PATH .. MAM.db.global.library[currentMacro].texture
				else
					return MAM.MACRO_ICON_PATH .. MAM.MACRO_DEFAULT_ICON
				end
			end,
			imageCoords = {0.08, 0.92, 0.08, 0.92},
			imageWidth = 30,
			imageHeight = 30,
			func = function()
			    InterfaceOptionsFrame:Hide()
				HideUIPanel(GameMenuFrame)
				MAM:IconPopUp(MAM.db.global.library[currentMacro].texture, currentMacro)
			end,
		},
		space11 = {
			type = "description",
			order = 8,
			name = " ",
			width = "full",
		},
		edit = {
			type = "input",
			order = 9,
			name = function()
				local characterNumber = MacroBodySize()
				return L["Macro edit box | Character number: "] .. MAM.NORMAL_COLOR .. characterNumber .. "\/" .. 255
			end,
			width = "full",
			multiline = 10,
			disabled = function()
				return currentMacro == 0
			end,
			validate = MacroBodySizeValidate,
			get = function()
				if currentMacro ~= 0 then
					return MAM.db.global.library[currentMacro].body
				else
					return ""
				end
			end,
			set = function(info, value)
				MAM.db.global.library[currentMacro].body = value
				for i = 1, MAM.db.profile.general.macroNumber do
					if MAM.db.profile.currentMacros[i] == currentMacro then
						MAM:UpdateIgMacro(i, MAM.db.global.library[currentMacro].name, "body", MAM.db.global.library[currentMacro].body)
					end
				end
			end,
		},
		space2 = {
			type = "description",
			order = 20,
			name = " ",
			width = "full",
		},
		descMakeRealMacro = {
			type = "description",
			order = 30,
			name = MAM.NORMAL_COLOR .. L["Use these options to create an in game macro from the above selected macro:"],
			fontSize = "medium",
		},
		perCharac = {
			type = "toggle",
			order = 40,
			disabled = function()
				return currentMacro == 0
			end,
			name = L["Character specific"],
			desc = L["With this option enabled, myAutoMacro will attempt to create the macro in your character specific storage location. This will fail if it's full."],
			get = function()
				return MAM.db.profile.general.perChar
			end,
			set = function(info, value)
				MAM.db.profile.general.perChar = value
			end,
		},
		createRealMacro = {
			type = "execute",
			order = 50,
			disabled = function()
				return currentMacro == 0
			end,
			name = L["Create macro"],
			func = function()
				local perChar, hasRoom
				local numMacroGlobal, numMacroPerChar = GetNumMacros()
				
				if numMacroPerChar < 18 and MAM.db.profile.general.perChar then
					perChar = true
					hasRoom = true
				elseif numMacroGlobal < 36 then
					hasRoom = true
				end
				
				if hasRoom then
					CreateMacro(MAM.db.global.library[currentMacro].name, MAM.db.global.library[currentMacro].texture, MAM.db.global.library[currentMacro].body, perChar)
					if perChar then
						print(format(MAM.FORMATED_ADDON_NAME .. L["The macro %s has been created in you character specific storage space"], HlString(MAM.db.global.library[currentMacro].name)))
					else
						print(format(MAM.FORMATED_ADDON_NAME .. L["The macro %s has been created in the global storage space"], HlString(MAM.db.global.library[currentMacro].name)))
					end
				else
					print(MAM.FORMATED_ADDON_NAME .. L["You don't have enough space to create any more macro. Please make room before attempting again."])
				end
			end
		},
		space3 = {
			type = "description",
			order = 100,
			name = " ",
			width = "full",
		},
		space31 = {
			type = "description",
			order = 110,
			name = " ",
			width = "full",
		},
		space32 = {
			type = "description",
			order = 120,
			name = " ",
			width = "full",
		},
		reset = {
			type = "execute",
			order = 130,
			name = L["Clear library"],
			desc = L["Clears entirely the macro library. Be carefull, it cannot be undone."],
			confirm = true,
			confirmText = L["This will clear the macro library and reset automacros. Are you sure?"],
			func = function()
				-- clears automacros body and resets current macros
				for i = 1, MAM.db.profile.general.macroNumber do
					if MAM.db.profile.currentMacros[i].name then
						EditMacro(MAM.MACRO_PREFIX .. i .. MAM.MACRO_SUFFIX .. MAM.db.global.library[MAM.db.profile.currentMacros[i]].name, MAM.MACRO_PREFIX .. i .. MAM.MACRO_SUFFIX, MAM.MACRO_DEFAULT_ICON, "")
					end
				end
				for i = 1, MAM.MAX_AUTOMACRO do
					MAM.db.profile.currentMacros[i] = ""
				end

				-- wipes library
				wipe(MAM.db.global.library)
				print(MAM.FORMATED_ADDON_NAME .. L["Macro library has been erased."])
				
				-- clears all associations
				MAM:MonsieurPropre("macro")
				
				-- reset current macro
				currentMacro = 0
			end,
		},
	}
	
	return args
end

-- automacro tabs creation
function MAM:AutomacrosOptions()
	local args = {
		desc = {
			type = "description",
			order = 1,
			name = MAM.NORMAL_COLOR .. L["descSelect"],
			width = "full",
		},
		bigSpace = {
			type = "description",
			order = 5,
			name = " ",
			width = "full",
		},
	}
	
	for autoIndex = 1, MAM.MAX_AUTOMACRO do
		-- locals
		local currentZoneType, currentAsso = 1, 1
		local currentZone, currentNPC, currentBoss, currentMacroForZone, currentMacroForNPC, currentMacroForBoss = -100, -100, -100, -100, -100, -100 -- we use negative indexes for addon internal stuff as blizz doesn't seem to use them
		local redundantIDs = {} -- when several npc are used to detect an encounter, their IDs are stored here
		
		-- options
		args["auto" .. autoIndex] = {
			type = "group",
			order = 10 + autoIndex,
			name = "Automacro" .. autoIndex,
			hidden = function(info)
				if info[#info] == "auto" .. autoIndex and tonumber(string.sub(info[#info], -1)) > MAM.db.profile.general.macroNumber then
					return true
				else
					return false
				end
			end,
			args = {
				assoSelect = {
					type = "select",
					order = 0,
					name = L["Associate a macro to:"],
					values = function()
						list = {L["Boss"], L["NPC"], L["Zone"]}
						return list
					end,
					get = function()
						return currentAsso
					end,
					set = function(info, value)
						currentAsso = tonumber(value)
						currentZoneType = 1
						currentBoss, currentNPC, currentZone, currentMacroForBoss, currentMacroForNPC, currentMacroForZone = -100, -100, -100, -100, -100, -100
					end,
				},
				zoneTypeSelect = {
					type = "select",
					order = 20,
					name = function()
						local name
						if currentAsso == 1 then
							name = L["Select an instance type"]
						else
							name = L["Select a zone type"]
						end
						
						return name
					end,
					values = function()
						local list = {}
						for i, v in ipairs(MAM:GetZoneType()) do
							list[i] = v
						end
						if currentAsso == 1 then
							list[4] = nil
							list[5] = nil
							list[6] = nil
							list[7] = nil
						end
							
						return list
					end,
					get = function(info)
						return currentZoneType
					end,
					set = function(info, value)
						currentZoneType = tonumber(value)
						currentZone = -100
					end,	
				},
				zoneSelect = {
					type = "select",
					order = 30,
					disabled = function()
						return currentZoneType == 1
					end,
					name = L["Select a zone"],
					desc = L["Zone to which associate a macro or where a NPC is located."],
					values = function()
						local list = {}
						for _, v in ipairs(MAM:FetchZones(currentZoneType)) do
							list[v] = MAM:FetchZoneName(v)
							if v == -90 and (currentAsso == 1 or currentAsso == 3) then
								list[v] = nil
							end
						end
						
						return list
					end,
					get = function(info)
						if MAM.zonedbHasChanged then
							currentZone = -100
							MAM.zonedbHasChanged = false
						end
						return currentZone
					end,
					set = function(info, value)
						if tonumber(value) ~= currentZone then
							currentBoss, currentNPC, currentMacroForBoss, currentMacroForNPC = -100, -100, -100, -100
						end
						currentZone = tonumber(value)
					end,	
				},
				space1 = {
					type = "description",
					order = 40,
					name = " ",
					width = "full",
				},
				bosses = {
					type = "group",
					order = 50,
					inline = true,
					hidden = function()
						return currentAsso ~= 1
					end,
					args = {
						header = {
							type = "header",
							order = 0,
							name = L["Boss association"],
						},
						select = {
							type = "select",
							order = 10,
							disabled = function()
								return currentZone == -100
							end,
							name = L["Select an encounter"],
							desc = L["The encounter to which associate a macro."],
							values = function()
								local list = MAM:GetBoss(currentZone)
								local compare = {}
								for k, v in pairs(list) do
									if not compare[v] then
										compare[v] = k
									else
										redundantIDs[k] = compare[v]
										list[k] = nil
									end
								end
								list[-100] = L["None"]
							
								return list
							end,
							get = function()
								return currentBoss
							end,
							set = function(info, value)
								currentBoss = tonumber(value)
								if value == -100 then
									currentMacroForBoss = -100
								end
							end,	
						},
						space1 = {
							type = "description",
							order = 15,
							name = " ",
							width = "full",
						},
						recap = {
							type = "description",
							order = 20,
							width = "full",
							fontSize = "medium",
							name = function()
								local libID, returnedText
								local bossName = MAM:GetBossName(currentZone, currentBoss)
								
								-- check if a macro is associated
								libID = MAM.db.profile.macro[autoIndex].npc[currentBoss]
								
								-- returned text
								if not libID then
									returnedText = format(L["nomacro"], HlString(L["No macro"]), L[" is currently associated to your selection."])
								else
									returnedText = format(MAM.NORMAL_COLOR .. L["Each time you will detect %s, Automacro%d will be switched to %s."], HlString(bossName), autoIndex, HlString(MAM.db.global.library[libID].name))
								end
								
								return returnedText
							end,
						},
						space2 = {
							type = "description",
							order = 30,
							name = " ",
							width = "full",
						},
						macroSelect = {
							type = "select",
							order = 40,
							disabled = function()
								return currentZoneType == "none" or currentZone == -100 or currentBoss == -100
							end,
							name = L["Select a macro"],
							values = function()
								local list = {}
								for i = 1, #MAM.db.global.library do
									list[i] = MAM.db.global.library[i].name
								end
								list[-100] = L["None"]

								return list
							end,
							get = function(info)
								return currentMacroForBoss
							end,
							set = function(info, value)
								currentMacroForBoss = value
							end,
						},
						space3 = {
							type = "description",
							order = 50,
							name = " ",
							width = "full",
						},
						association = {
							type = "execute",
							order = 60,
							disabled = function()
								return currentZoneType == "none" or currentZone == -100 or currentMacroForBoss == -100 or currentMacroForBoss == MAM.db.profile.macro[autoIndex].npc[currentBoss]
							end,
							name = L["Associate!"],
							func = function()					
								MAM.db.profile.macro[autoIndex].npc[currentBoss] = currentMacroForBoss
								for k, v in pairs(redundantIDs) do
									if redundantIDs[k] == currentBoss then
										MAM.db.profile.macro[autoIndex].npc[k] = currentMacroForBoss
									end
								end
								if UnitExists("target") then
									MAM:IdentifyNPC(UnitGUID("target"), "player")
								end
							end,
						},
						clear = {
							type = "execute",
							order = 70,
							disabled = function()
								return currentZoneType == "none" or currentZone == -100 or not MAM.db.profile.macro[autoIndex].npc[currentBoss]
							end,
							name = L["Clear!"],
							func = function()
								MAM.db.profile.macro[autoIndex].npc[currentBoss] = nil
								for k, v in pairs(redundantIDs) do
									if redundantIDs[k] == currentBoss then
										MAM.db.profile.macro[autoIndex].npc[k] = nil
									end
								end
							end,
						},
					},
				},
				npc = {
					type = "group",
					order = 60,
					inline = true,
					hidden = function()
						return currentAsso ~= 2
					end,
					args = {
						header = {
							type = "header",
							order = 0,
							name = L["NPC association"],
						},
						select = {
							type = "select",
							order = 10,
							disabled = function()
								return currentZoneType == 1 or currentZone == -100
							end,
							name = L["Select a NPC"],
							desc = L["The NPC to which associate a macro."],
							values = function()
								local list = {}
								for k, v in pairs(MAM:FetchNPCs(currentZone)) do
									list[k] = v
								end
								
								return list
							end,
							get = function()
								if MAM.NPCdbHasChanged then
									currentNPC = -100
									MAM.NPCdbHasChanged = false
								end
								return currentNPC
							end,
							set = function(info, value)
								currentNPC = tonumber(value)
								if value == -100 then
									currentMacroForNPC = -100
								end
							end,
						},
						space1 = {
							type = "description",
							order = 15,
							name = " ",
							width = "full",
						},
						recap = {
							type = "description",
							order = 20,
							width = "full",
							fontSize = "medium",
							name = function()
								local libID, returnedText
								local npcName = MAM:FetchNPCName(currentZone, currentNPC)
								
								-- check if a macro is associated
								libID = MAM.db.profile.macro[autoIndex].npc[currentNPC]
								
								-- returned text
								if not libID then
									returnedText = format(L["nomacro"], HlString(L["No macro"]), L[" is currently associated to your selection."])
								else
									returnedText = format(MAM.NORMAL_COLOR .. L["Each time you will detect %s, Automacro%d will be switched to %s."], HlString(npcName), autoIndex, HlString(MAM.db.global.library[libID].name))
								end
								
								return returnedText
							end,
						},
						space2 = {
							type = "description",
							order = 30,
							name = " ",
							width = "full",
						},
						macroSelect = {
							type = "select",
							order = 40,
							disabled = function()
								return currentZoneType == "none" or currentZone == -100 or currentNPC == -100
							end,
							name = L["Select a macro"],
							values = function()
								local list = {}
								for i = 1, #MAM.db.global.library do
									list[i] = MAM.db.global.library[i].name
								end
								list[-100] = L["None"]

								return list
							end,
							get = function(info)
								return currentMacroForNPC
							end,
							set = function(info, value)
								currentMacroForNPC = value
							end,
						},
						space3 = {
							type = "description",
							order = 50,
							name = " ",
							width = "full",
						},
						association = {
							type = "execute",
							order = 60,
							disabled = function()
								return currentZoneType == "none" or currentZone == -100 or currentMacroForNPC == -100 or currentMacroForNPC == MAM.db.profile.macro[autoIndex].npc[currentNPC]
							end,
							name = L["Associate!"],
							func = function()					
								MAM.db.profile.macro[autoIndex].npc[currentNPC] = currentMacroForNPC
								if UnitExists("target") then
									MAM:IdentifyNPC(UnitGUID("target"), "player")
								end
							end,
						},
						clear = {
							type = "execute",
							order = 70,
							disabled = function()
								return currentZoneType == "none" or currentZone == -100 or not MAM.db.profile.macro[autoIndex].npc[currentNPC]
							end,
							name = L["Clear!"],
							func = function()
								MAM.db.profile.macro[autoIndex].npc[currentNPC] = nil
							end,
						},
					},
				},
				zone = {
					type = "group",
					order = 60,
					inline = true,
					hidden = function()
						return currentAsso ~= 3
					end,
					args = {
						header = {
							type = "header",
							order = 0,
							name = L["Zone association"],
						},
						space1 = {
							type = "description",
							order = 8,
							name = " ",
							width = "full",
						},
						recap = {
							type = "description",
							order = 10,
							width = "full",
							fontSize = "medium",
							name = function()
								local zoneName, libID, returnedText
								if currentZone >= 0 then
									zoneName = GetMapNameByID(currentZone)
								elseif currentZone >= -80 and currentZone <= -71 then
									for name, zoneID in pairs(MAM:GetArenaTablePerLocale(MAM.locale)) do
										if zoneID == currentZone then
											zoneName = name
										end
									end
								end
																	
								-- check if a macro is associated
								libID = MAM.db.profile.macro[autoIndex].zone[currentZone]
								
								-- returned text
								if not libID then
									returnedText = format(L["nomacro"], HlString(L["No macro"]), L[" is currently associated to your selection."])
								else
									returnedText = format(MAM.NORMAL_COLOR .. L["Each time you will enter in %s, Automacro%d will be switched to %s."], HlString(zoneName), autoIndex, HlString(MAM.db.global.library[libID].name))
								end
								
								return returnedText
							end,
						},
						space2 = {
							type = "description",
							order = 20,
							name = " ",
							width = "full",
						},
						macroSelect = {
							type = "select",
							order = 30,
							disabled = function()
								return currentZoneType == "none" or currentZone == -100
							end,
							name = L["Select a macro"],
							values = function()
								local list = {}
								for i = 1, #MAM.db.global.library do
									list[i] = MAM.db.global.library[i].name
								end
								list[-100] = L["None"]

								return list
							end,
							get = function(info)
								return currentMacroForZone
							end,
							set = function(info, value)
								currentMacroForZone = value
							end,
						},
						space3 = {
							type = "description",
							order = 40,
							name = " ",
							width = "full",
						},
						association = {
							type = "execute",
							order = 50,
							disabled = function()
								return currentZoneType == "none" or currentZone == -100 or currentMacroForZone == -100 or currentMacroForZone == MAM.db.profile.macro[autoIndex].zone[currentZone]
							end,
							name = L["Associate!"],
							func = function()					
								MAM.db.profile.macro[autoIndex].zone[currentZone] = currentMacroForZone
								MAM:IdentifyZone("silent")
							end,
						},
						clear = {
							type = "execute",
							order = 60,
							disabled = function()
								return currentZoneType == "none" or currentZone == -100 or not MAM.db.profile.macro[autoIndex].zone[currentZone]
							end,
							name = L["Clear!"],
							func = function()
								MAM.db.profile.macro[autoIndex].zone[currentZone] = nil
							end,
						},
					},
				},
			},
		}	
	end

	return args
end

-- comm options
function MAM:ComOptions()
	local currentMacro = 0
	local currentCanal = "GUILD"
	local currentPlayerToSend
	local currentPlayerToBan
	local currentPlayerToDeban = "NONE"
	
	local arg = {
		enableIncomming = {
			type = "toggle",
			order = 0,
			name = L["Enable incomming"],
			desc = L["Toggle to enable incomming macros from other players"],
			get = function()
				return MAM.db.profile.general.incomming
			end,
			set = function(info, value)
				MAM.db.profile.general.incomming = value
			end,
		},
		space2 = {
			type = "description",
			order = 30,
			name = " ",
			width = "full",
		},
		space21 = {
			type = "description",
			order = 31,
			name = " ",
			width = "full",
		},
		descComm = {
			type = "description",
			order = 30,
			name = MAM.NORMAL_COLOR .. L["Use these tools to send the selected macro:"],
			fontSize = "medium",
			width = "full",
		},
		select = {
			type = "select",
			order = 40,
			name = L["Select a macro"],
			values = function()
				local list = {}
				for i = 1, #MAM.db.global.library do
					list[i] = MAM.db.global.library[i].name
				end
				list[0] = L["None"]

				return list
			end,
			get = function()
				return currentMacro
			end,
			set = function(info, value)
				currentMacro = value
			end,
		},
		canalSelect = {
			type = "select",
			order = 50,
			disabled = function()
				return currentMacro == 0
			end,
			name = L["Select a canal"],
			values = function()
				local list = {
					PARTY = L["PARTY"],
					RAID = L["RAID"],
					GUILD = L["GUILD"],
					BATTLEGROUND = L["BATTLEGROUND"],
					WHISPER = L["WHISPER TO"],
				}
				return list
			end,
			get = function()
				return currentCanal
			end,
			set = function(info, value)
				currentCanal = value
				if currentCanal ~= "WHISPER" then
					currentPlayerToSend = nil
				end
			end,
		},
		playerInput = {
			type = "input",
			order = 60,
			disabled = function()
				return currentMacro == 0 or currentCanal ~= "WHISPER"
			end,
			name = L["Enter player name"],
			get = function()
				return currentPlayerToSend
			end,
			set = function(info, value)
				currentPlayerToSend = value
			end,
		},
		space3 = {
			type = "description",
			order = 70,
			name = " ",
			width = "full",
		},
		sendBtn = {
			type = "execute",
			order = 80,
			disabled = function()
				return currentMacro == 0 or currentCanal == "WHISPER" and not currentPlayerToSend
			end,
			name = L["Send macro!"],
			func = function()
				local macroName = MAM.db.global.library[currentMacro].name
				local macroIcon = MAM.db.global.library[currentMacro].texture
				local macroBody = MAM.db.global.library[currentMacro].body				
				MAM:SendCommMessage(MAM.ADDON_PREFIX, MAM:CodeMacro(macroName, macroIcon, macroBody), currentCanal, currentPlayerToSend)
				if currentPlayerToSend then
					print(MAM.FORMATED_ADDON_NAME .. format(L["You have sent the macro %s to %s."], HlString(macroName), HlString(currentPlayerToSend)))
				else
					print(MAM.FORMATED_ADDON_NAME .. format(L["You have sent the macro %s over the %s canal."], HlString(macroName), HlString(L[currentCanal])))
				end
			end,
		},
		space4 = {
			type = "description",
			order = 90,
			name = " ",
			width = "full",
		},
		space41 = {
			type = "description",
			order = 100,
			name = " ",
			width = "full",
		},
		descBan = {
			type = "description",
			order = 110,
			name = MAM.NORMAL_COLOR .. L["Use these tools to add or remove a player from the blacklist:"],
			fontSize = "medium",
			width = "full",
		},
		addPlayerInput = {
			type = "input",
			order = 120,
			name = L["Enter the name of a player"],
			validate = function(info, value)
				if value ~= "" and not string.find(value, "%s") and not string.find(value, "%d") and not string.find(value, "%p") then
					return true
				else
					print(MAM.FORMATED_ADDON_NAME .. L["This name is not valid."])
					return false
				end
			end,
			get = function()
				return currentPlayerToBan
			end,
			set = function(info, value)
				currentPlayerToBan = string.upper(string.sub(value, 1, 1)) .. string.sub(value, 2)
			end,
		},
		addPlayerExec = {
			type = "execute",
			order = 130,
			disabled = function()
				return not currentPlayerToBan
			end,
			name = L["Add player!"],
			func = function()
				MAM.db.global.blacklisted[currentPlayerToBan] = currentPlayerToBan
				print(MAM.FORMATED_ADDON_NAME .. format(L["%s has been added to the blacklist. You will not receive anymore macro from him."], HlString(currentPlayerToBan)))
				currentPlayerToBan = nil
			end,
		},
		space5 = {
			type = "description",
			order = 140,
			name = " ",
			width = "full",
		},
		playerDebanSelect = {
			type = "select",
			order = 150,
			name = L["Select a player"],
			desc = L["Select a player to remove from the blacklist"],
			values = function()
				local list = DeepCopy(MAM.db.global.blacklisted)
				list["NONE"] = L["None"]
				return list
			end,
			get = function()
				return currentPlayerToDeban
			end,
			set = function(info, value)
				currentPlayerToDeban = value
			end,
		},
		playerDebanExec = {
			type = "execute",
			order = 160,
			disabled = function()
				return currentPlayerToDeban == "NONE"
			end,
			name = L["Remove player!"],
			func = function()
				MAM.db.global.blacklisted[currentPlayerToDeban] = nil
				print(MAM.FORMATED_ADDON_NAME .. format(L["%s has been removed from the blacklist. You are now able to receive macro from him."], HlString(currentPlayerToDeban)))
				currentPlayerToDeban = "NONE"
			end,
		},
	}
	
	return arg
end

-- npc options
function MAM:NPCOptions()
	local currentZoneType, currentZone, currentNPCToDelete = 1, -100, -100
	local currentNPCID, currentNPCName = "", ""
	local NPCIDIsValid, NPCNameIsValid = false, false
	local moveWho, toZone, moveToType = -100, -100, 1
	
	local args = {
		scanning = {
			type = "toggle",
			order = 0,
			name = L["Enable NPC ID printing"],
			desc = L["Enabling this options will allow myAutoMacro to print the ID of every NPC you mouseover or target in your chat."],
			get = function()
				return MAM.db.profile.general.printNPCID
			end,
			set = function(info, value)
				MAM.db.profile.general.printNPCID = value
				if UnitExists("target") then
					MAM:IdentifyNPC(UnitGUID("target"), "player")
				end
			end,
		},
		space1 = {
			type = "description",
			order = 10,
			name = " ",
			width = "full",
		},
		space11 = {
			type = "description",
			order = 15,
			name = " ",
			width = "full",
		},
		addMessage = {
			type = "description",
			order = 20,
			name =  MAM.NORMAL_COLOR .. L["Use the tools below to add a NPC:"],
			fontSize = "medium",
		},
		zoneTypeSelect = {
			type = "select",
			order = 30,
			name = L["Select a zone type"],
			values = function()
				local list = {}
				for i, v in ipairs(MAM:GetZoneType()) do
					list[i] = v
				end
				return list
			end,
			get = function(info)
				return currentZoneType
			end,
			set = function(info, value)
				currentZoneType = tonumber(value)
				currentZone = -100
			end,	
		},
		zoneSelect = {
			type = "select",
			order = 40,
			disabled = function()
				return currentZoneType == 1
			end,
			name = L["Select a zone"],
			desc = L["Will store the custom NPC into this section. The NPC has not to be necessarily located in this zone in game. It's up to you!"],
			values = function()
				local list = {}
				for _, v in ipairs(MAM:FetchZones(currentZoneType)) do
					list[v] = MAM:FetchZoneName(v)
				end
				
				return list
			end,
			get = function(info)
				return currentZone
			end,
			set = function(info, value)
				currentZone = tonumber(value)
			end,	
		},
		space12 = {
			type = "description",
			order = 50,
			name = " ",
			width = "full",
		},
		inputID = {
			type = "input",
			order = 60,
			name = L["Enter NPC ID"],
			disabled = function()
				return currentZoneType == 1 or currentZone == -100
			end,
			validate = function(info, value)
				local value = tonumber(value)
				local isValid = false
				
				if value and type(value) == "number" and value ~= "" then
					isValid = true
					for zone in pairs(MAM.db.global.custom.npc) do
						for npc in pairs(MAM.db.global.custom.npc[zone]) do
							if value == npc then
								print(MAM.FORMATED_ADDON_NAME .. L["This ID already exists!"])
								isValid = false
							end
						end
					end
				else
					print(MAM.FORMATED_ADDON_NAME .. L["You must enter a number!"])
					isValid = false
				end
				
				NPCIDIsValid = isValid
				return isValid
			end,
			get = function()
				return tostring(currentNPCID)
			end,
			set = function(info, value)
				currentNPCID = tonumber(value)
			end,
		},
		inputName = {
			type = "input",
			order = 70,
			name = L["Enter NPC name"],
			disabled = function()
				return currentZoneType == 1 or currentZone == -100
			end,
			validate = function(info, value)
				local value = tostring(value)
				local isValid = false
				
				if value and type(value) == "string" and value ~= "" then
					isValid = true
					for zone in pairs(MAM.db.global.custom.npc) do
						for npc in pairs(MAM.db.global.custom.npc[zone]) do
							if value == MAM.db.global.custom.npc[zone][npc] then
								print(MAM.FORMATED_ADDON_NAME .. L["This name already exists. Please choose another one."])
								isValid = false
							end
						end
					end
				else
					print(MAM.FORMATED_ADDON_NAME .. L["You must enter a valid name!"])
					isValid = false
				end
				
				NPCNameIsValid = isValid
				return isValid
			end,
			get = function()
				return currentNPCName
			end,
			set = function(info, value)
				currentNPCName = tostring(value)
			end,
		},
		space13 = {
			type = "description",
			order = 75,
			name = " ",
			width = "full",
		},
		add = {
			type = "execute",
			order = 80,
			disabled = function()
				return currentZoneType == 1 or currentZone == -100 or not (NPCIDIsValid and NPCNameIsValid)
			end,
			name = L["Add!"],
			func = function()
				MAM.db.global.custom.npc[currentZone] = MAM.db.global.custom.npc[currentZone] or {}
				MAM.db.global.custom.npc[currentZone][currentNPCID] = currentNPCName
				
				-- chat message
				print(HlString(currentNPCName) .. L[" has been added to myAutoMacro with the ID "] .. HlString(currentNPCID))
				
				-- reset
				currentNPCID = ""
				currentNPCName = ""
				NPCIDIsValid = false
				NPCNameIsValid = false
				MAM.NPCdbHasChanged = true
			end,
		},
		space4 = {
			type = "description",
			order = 90,
			name = " ",
			width = "full",
		},
		space41 = {
			type = "description",
			order = 91,
			name = " ",
			width = "full",
		},
		moveMessage = {
			type = "description",
			order = 92,
			name =  MAM.NORMAL_COLOR .. L["Use the tools below to change the zone of a custom NPC:"],
			fontSize = "medium",
		},
		moveWho = {
			type = "select",
			order = 93,
			name = L["Select a NPC to move"],
			values = function()
				local list = {}
				for zone in pairs(MAM.db.global.custom.npc) do
					AddCopy(MAM:FetchNPCs(zone), list)
				end
				list[-100] = L["None"]
				
				return list
			end,
			get = function()
				return moveWho
			end,
			set = function(info, value)
				moveWho = value
			end,
		},
		moveToZoneType = {
			type = "select",
			order = 94,
			disabled = function()
				return moveWho == -100
			end,
			name = L["Move to zone type:"],
			values = function()
				local list = {}
				for i, v in ipairs(MAM:GetZoneType()) do
					list[i] = v
				end
				return list
			end,
			get = function()
				return moveToType
			end,
			set = function(info, value)
				moveToType = tonumber(value)
				toZone = -100
			end,
		},
		moveToZone = {
			type = "select",
			order = 95,
			disabled = function()
				return moveWho == -100 or moveToType == 1
			end,
			name = L["Move to:"],
			values = function()
				local list = {}
				for _, v in ipairs(MAM:FetchZones(moveToType)) do
					list[v] = MAM:FetchZoneName(v)
				end
				
				return list
			end,
			get = function()
				return toZone
			end,
			set = function(info, value)
				toZone = value
			end,
		},
		space42 = {
			type = "description",
			order = 96,
			name = " ",
			width = "full",
		},
		moveExec = {
			type = "execute",
			order = 97,
			name = L["Move!"],
			disabled = function()
				return moveWho == -100 or toZone == -100 or moveToType == 1
			end,
			func = function()
				local fromZone
				for zone in pairs(MAM.db.global.custom.npc) do
					for npc in pairs(MAM.db.global.custom.npc[zone]) do
						if npc == moveWho then
							fromZone = zone
						end
					end
				end
				if fromZone ~= toZone then
					MAM.db.global.custom.npc[toZone] = MAM.db.global.custom.npc[toZone] or {}
					MAM.db.global.custom.npc[toZone][moveWho] = MAM.db.global.custom.npc[fromZone][moveWho]
					MAM.db.global.custom.npc[fromZone][moveWho] = nil
					MAM.NPCdbHasChanged = true
					print(MAM.FORMATED_ADDON_NAME .. format(L["%s has been moved from %s to %s."], HlString(MAM.db.global.custom.npc[toZone][moveWho]), HlString(MAM:FetchZoneName(fromZone)), HlString(MAM:FetchZoneName(toZone))))
				else
					print(MAM.FORMATED_ADDON_NAME .. format(L["%s has been kept in %s."], HlString(MAM.db.global.custom.npc[toZone][moveWho]), HlString(MAM:FetchZoneName(fromZone))))
				end
			end,
		},
		space5 = {
			type = "description",
			order = 98,
			name = " ",
			width = "full",
		},
		space51 = {
			type = "description",
			order = 99,
			name = " ",
			width = "full",
		},
		deleteMessage = {
			type = "description",
			order = 100,
			name =  MAM.NORMAL_COLOR .. L["Use the tools below to delete a custom NPC:"],
			fontSize = "medium",
		},
		npcToDelSelect = {
			type = "select",
			order = 110,
			name = L["Select a NPC to delete"],
			desc = L["Select a NPC to delete from the database. Attention, this will affect every automacro associated to this NPC and all your characters."],
			values = function()
				local list = {}
				for zone in pairs(MAM.db.global.custom.npc) do
					AddCopy(MAM:FetchNPCs(zone), list)
				end
				list[-100] = L["None"]
				
				return list
			end,
			get = function()
				return currentNPCToDelete
			end,
			set = function(info, value)
				currentNPCToDelete = tonumber(value)
			end,
		},
		space52 = {
			type = "description",
			order = 115,
			name = " ",
			width = "full",
		},
		delete = {
			type = "execute",
			order = 120,
			name = L["Delete!"],
			disabled = function()
				return currentNPCToDelete == -100
			end,
			confirm = true,
			confirmText = L["This NPC and every possible association will be lost. Are you sure?"],
			func = function()
				local name
				-- clears npc entry
				for zone in pairs(MAM.db.global.custom.npc) do
					if MAM.db.global.custom.npc[zone][currentNPCToDelete] then
						name = MAM:FetchNPCName(zone, currentNPCToDelete)
						MAM.db.global.custom.npc[zone][currentNPCToDelete] = nil
					end
				end
				
				-- clears associations
				MAM:MonsieurPropre("npc")
				
				print(HlString(name) .. L[" has been deleted from the database."])
				currentNPCToDelete = -100
				moveWho = -100
				MAM.NPCdbHasChanged = true
			end,
		},
		space6 = {
			type = "description",
			order = 130,
			name = " ",
			width = "full",
		},
		space61 = {
			type = "description",
			order = 131,
			name = " ",
			width = "full",
		},
		deleteAll = {
			type = "execute",
			order = 140,
			name = L["Delete all NPC"],
			confirm = true,
			confirmText = L["All NPC and every possible association will be lost. Are you sure?"],
			func = function()
				-- deletes db
				for _, entries in pairs(MAM.db.global.custom.npc) do
					for npc in pairs(entries) do
						entries[npc] = nil
					end
				end

				-- clears associations
				MAM:MonsieurPropre("npc")

				currentNPCToDelete = -100
				MAM.NPCdbHasChanged = true
				print(MAM.HIGHLIGHT_COLOR .. L["All custom NPC have been deleted from the database."])
			end,
		},
	}
		
	return args
end
	
-- zone options
function MAM:ZoneOptions()
	local currentZoneType, currentZoneToDelete = 1, -100
	local currentZoneID, currentZoneName = "", L["None"]
	local ZoneIDIsValid = false
	local moveWhat, moveToType = -100, 1
	
	local args = {
		scanning = {
			type = "toggle",
			order = 0,
			name = L["Enable Zone ID printing"],
			desc = L["Enabling this options will allow myAutoMacro to print the ID of every zone you are visiting in your chat."],
			get = function()
				return MAM.db.profile.general.printZoneID
			end,
			set = function(info, value)
				MAM.db.profile.general.printZoneID = value
				MAM:IdentifyZone()
			end,
		},
		space1 = {
			type = "description",
			order = 10,
			name = " ",
			width = "full",
		},
		space11 = {
			type = "description",
			order = 15,
			name = " ",
			width = "full",
		},
		addMessage = {
			type = "description",
			order = 20,
			name =  MAM.NORMAL_COLOR .. L["Use the tools below to add a zone:"],
			fontSize = "medium",
		},
		zoneTypeSelect = {
			type = "select",
			order = 30,
			name = L["Select a zone type"],
			values = function()
				local list = {}
				for i, v in ipairs(MAM:GetZoneType()) do
					list[i] = v
				end
				return list
			end,
			get = function(info)
				return currentZoneType
			end,
			set = function(info, value)
				currentZoneType = tonumber(value)
			end,	
		},
		space12 = {
			type = "description",
			order = 50,
			name = " ",
			width = "full",
		},
		inputID = {
			type = "input",
			order = 60,
			name = L["Enter zone ID"],
			disabled = function()
				return currentZoneType == 1
			end,
			validate = function(info, value)
				local value = tonumber(value)
				local isValid = false
				
				if value and type(value) == "number" and value ~= "" then
					if GetMapNameByID(value) then
						isValid = true
					else
						print(MAM.FORMATED_ADDON_NAME .. L["This ID do not correspond to any zone of the game. Please enter a corredct ID."])
					end
					for sections in pairs(MAM.db.global.custom.zone) do
						for _, zone in pairs(MAM.db.global.custom.zone[sections]) do
							if value == zone then
								print(MAM.FORMATED_ADDON_NAME .. L["This ID already exists!"])
								isValid = false
							end
						end
					end
				else
					print(MAM.FORMATED_ADDON_NAME .. L["You must enter a number!"])
					isValid = false
				end
				
				ZoneIDIsValid = isValid
				return isValid
			end,
			get = function()
				return tostring(currentZoneID)
			end,
			set = function(info, value)
				currentZoneID = tonumber(value)
				currentZoneName = GetMapNameByID(currentZoneID)
			end,
		},
		zoneName = {
			type = "input",
			order = 70,
			disabled = true,
			name = MAM.NORMAL_COLOR  .. L["Zone name:"],
			get = function()
				return MAM.HIGHLIGHT_COLOR .. currentZoneName
			end,
		},
		space13 = {
			type = "description",
			order = 75,
			name = " ",
			width = "full",
		},
		add = {
			type = "execute",
			order = 80,
			disabled = function()
				return currentZoneType == 1 or not ZoneIDIsValid
			end,
			name = L["Add!"],
			func = function()
				MAM.db.global.custom.zone[currentZoneType] = MAM.db.global.custom.zone[currentZoneType] or {}
				table.insert(MAM.db.global.custom.zone[currentZoneType], currentZoneID)
				
				-- chat message
				print(HlString(currentZoneName) .. L[" has been added to myAutoMacro with the ID "] .. HlString(currentZoneID))
				
				-- reset
				currentZoneID = ""
				currentZoneName = L["None"]
				ZoneIDIsValid = false
				MAM.zonedbHasChanged = true
			end,
		},
		space4 = {
			type = "description",
			order = 90,
			name = " ",
			width = "full",
		},
		space41 = {
			type = "description",
			order = 91,
			name = " ",
			width = "full",
		},
		moveMessage = {
			type = "description",
			order = 92,
			name =  MAM.NORMAL_COLOR .. L["Use the tools below to change the type of a zone:"],
			fontSize = "medium",
		},
		moveWhat = {
			type = "select",
			order = 93,
			name = L["Select a zone to move"],
			values = function()
				local list = {}
				for zonetype in pairs(MAM.db.global.custom.zone) do
					for _, zone in ipairs(MAM.db.global.custom.zone[zonetype]) do
						list[zone] = GetMapNameByID(zone)
					end
				end
				list[-100] = L["None"]
				
				return list
			end,
			get = function()
				return moveWhat
			end,
			set = function(info, value)
				moveWhat = value
			end,
		},
		moveToZoneType = {
			type = "select",
			order = 94,
			disabled = function()
				return moveWhat == -100
			end,
			name = L["Move to zone type:"],
			values = function()
				local list = {}
				for i, v in ipairs(MAM:GetZoneType()) do
					list[i] = v
				end
				return list
			end,
			get = function()
				return moveToType
			end,
			set = function(info, value)
				moveToType = tonumber(value)
			end,
		},
		space42 = {
			type = "description",
			order = 96,
			name = " ",
			width = "full",
		},
		moveExec = {
			type = "execute",
			order = 97,
			name = L["Move!"],
			disabled = function()
				return moveWhat == -100 or moveToType == 1
			end,
			func = function()
				local fromZoneType
				for zonetype in pairs(MAM.db.global.custom.zone) do
					for entries, zone in ipairs(MAM.db.global.custom.zone[zonetype]) do
						if zone == moveWhat then
							fromZoneType = zonetype
							table.remove(MAM.db.global.custom.zone[zonetype], entries)
						end
					end
				end
				local zoneTypeTable = DeepCopy(MAM:GetZoneType())
				
				MAM.db.global.custom.zone[moveToType] = MAM.db.global.custom.zone[moveToType] or {}
				table.insert(MAM.db.global.custom.zone[moveToType], moveWhat)
				MAM.zonedbHasChanged = true	
				if fromZoneType ~= moveToType then
					print(MAM.FORMATED_ADDON_NAME .. format(L["%s has been moved from %s to %s."], HlString(MAM:FetchZoneName(moveWhat)), HlString(zoneTypeTable[fromZoneType]), HlString(zoneTypeTable[moveToType])))
				else
					print(MAM.FORMATED_ADDON_NAME .. format(L["%s has been kept in %s."], HlString(MAM:FetchZoneName(moveWhat)), HlString(zoneTypeTable[fromZoneType])))
				end
			end,
		},
		space5 = {
			type = "description",
			order = 98,
			name = " ",
			width = "full",
		},
		space51 = {
			type = "description",
			order = 99,
			name = " ",
			width = "full",
		},
		deleteMessage = {
			type = "description",
			order = 100,
			name =  MAM.NORMAL_COLOR .. L["Use the tools below to delete a custom zone:"],
			fontSize = "medium",
		},
		zoneToDelSelect = {
			type = "select",
			order = 110,
			name = L["Select a Zone to delete"],
			desc = L["Select a Zone to delete from the database. Attention, this will affect every automacro associated to this Zone and all your characters."],
			values = function()
				local list = {}
				for zonetype in pairs(MAM.db.global.custom.zone) do
					for _, zone in ipairs(MAM.db.global.custom.zone[zonetype]) do
						list[zone] = GetMapNameByID(zone)
					end
				end
				list[-100] = L["None"]
				
				return list
			end,
			get = function()
				return currentZoneToDelete
			end,
			set = function(info, value)
				currentZoneToDelete = tonumber(value)
			end,
		},
		space52 = {
			type = "description",
			order = 115,
			name = " ",
			width = "full",
		},
		delete = {
			type = "execute",
			order = 120,
			name = L["Delete!"],
			disabled = function()
				return currentZoneToDelete == -100
			end,
			confirm = true,
			confirmText = L["This zone and every possible association will be lost. Are you sure?"],
			func = function()
				-- deletes zone entry
				for zonetype in pairs(MAM.db.global.custom.zone) do
					for index, zone in ipairs(MAM.db.global.custom.zone[zonetype]) do
						if zone == currentZoneToDelete then
							table.remove(MAM.db.global.custom.zone[zonetype], index)
						end
					end
				end
				
				-- moves npcs in "anywhere" zone
				if MAM.db.global.custom.npc[currentZoneToDelete] then
					MAM.db.global.custom.npc[-90] = MAM.db.global.custom.npc[-90] or {}
					for npc in pairs(MAM.db.global.custom.npc[currentZoneToDelete]) do
						MAM.db.global.custom.npc[-90][npc] = MAM.db.global.custom.npc[currentZoneToDelete][npc]
						MAM.db.global.custom.npc[currentZoneToDelete][npc] = nil
					end
					MAM.db.global.custom.npc[currentZoneToDelete] = nil
				end
				
				-- clears associations
				MAM:MonsieurPropre("zone")				
				
				print(HlString(MAM:FetchZoneName(currentZoneToDelete)) .. L[" has been deleted from the database."])
				currentZoneToDelete = -100
				moveWhat = -100
				MAM.zonedbHasChanged = true
				MAM.NPCdbHasChanged = true
			end,
		},
		space6 = {
			type = "description",
			order = 130,
			name = " ",
			width = "full",
		},
		space61 = {
			type = "description",
			order = 131,
			name = " ",
			width = "full",
		},
		deleteAll = {
			type = "execute",
			order = 140,
			name = L["Delete all zones"],
			confirm = true,
			confirmText = L["All zones and every possible association will be lost. Are you sure?"],
			func = function()
				local wereCustomZones = {}
				-- resets db
				for _, entries in pairs(MAM.db.global.custom.zone) do
					for _, zoneID in ipairs(entries) do
						wereCustomZones[zoneID] = true
					end
					wipe(entries)
				end
				
				-- moves npcs in "anywhere" zone
				for zone in pairs(wereCustomZones) do
					if MAM.db.global.custom.npc[zone] then
						MAM.db.global.custom.npc[-90] = MAM.db.global.custom.npc[-90] or {}
						for npc in pairs(MAM.db.global.custom.npc[zone]) do
							MAM.db.global.custom.npc[-90][npc] = MAM.db.global.custom.npc[zone][npc]
							MAM.db.global.custom.npc[zone][npc] = nil
						end
						MAM.db.global.custom.npc[zone] = nil
					end
				end				

				-- clears associations
				MAM:MonsieurPropre("zone")				
				
				print(MAM.HIGHLIGHT_COLOR .. L["All custom zones have been deleted from the database."])
				currentZoneToDelete = -100
				moveWhat = -100
				MAM.zonedbHasChanged = true
				MAM.NPCdbHasChanged = true
			end,
		},
	}
		
	return args
end
	


-- Config pannel functions ----------------

-- load real options
function MAM:LoadOptions()
	wipe(MAM.options.args)
	MAM.options.args = MAM:Options()
	MAM.options.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(MAM.db)
end

-- fetch zones
function MAM:FetchZones(zoneType)
	local list = MAM:GetZone(zoneType)
	
	if MAM.db.global.custom.zone[zoneType] then
		for _, v in ipairs(MAM.db.global.custom.zone[zoneType]) do
			table.insert(list, v)
		end
	end

	return list
end

-- fetch zone names
function MAM:FetchZoneName(ID)
	if ID > 0 then
		return GetMapNameByID(ID)
	elseif ID == -90 then
		return L["Anywhere"]
	elseif ID >= -80 and ID <= -71 then
		for zoneName, zoneID in pairs(MAM:GetArenaTablePerLocale(MAM.locale)) do
			if zoneID == ID then
				return zoneName
			end
		end
	else
		return L["None"]
	end
end

-- fetch NPC
function MAM:FetchNPCs(zone)
	local list = {}
	
	if MAM.db.global.custom.npc[zone] then
		list = DeepCopy(MAM.db.global.custom.npc[zone])
	end
	list[-100] = L["None"]
	
	return list
end

-- fetch npc name
function MAM:FetchNPCName(zone, ID)
	if MAM.db.global.custom.npc[zone] then
		if MAM.db.global.custom.npc[zone][ID] then
			return MAM.db.global.custom.npc[zone][ID]
		end
	end
	return ""
end

-- update ig automacro when we edit the currently associated macro
function MAM:UpdateIgMacro(automacro, macroName, paramType, value)
	local macroNewName, macroNewIcon, macroNewBody
	if paramType == "name" then
		macroNewName = MAM.MACRO_PREFIX .. automacro .. MAM.MACRO_SUFFIX .. value
	elseif paramType == "icon" then
		macroNewIcon = value
	elseif paramType == "body" then
		macroNewBody = value
	else
		return
	end
	
	-- updating in combat shouldn't happen as this update is triggered by the user modifying the config, which is not allowed in combat
	if not InCombatLockdown() then
		EditMacro(MAM.MACRO_PREFIX .. automacro .. MAM.MACRO_SUFFIX .. macroName, macroNewName, macroNewIcon, macroNewBody)
	end
end

-- erase associations on every characters when we delete a macro or a custom npc/zone
function MAM:MonsieurPropre(typeToClear)
	for _, prof in pairs(MAM.db:GetProfiles()) do
		for i = 1, MAM.MAX_AUTOMACRO do
			if typeToClear == "macro" then
				if MAM.db.profiles[prof].macro[i] and MAM.db.profiles[prof].macro[i].zone then
					for zone, libID in pairs(MAM.db.profiles[prof].macro[i].zone) do
						if not MAM.db.global.library[libID] or MAM.db.global.library[libID] == "deleted" then
							MAM.db.profiles[prof].macro[i].zone[zone] = nil
						end
					end
				end
				if MAM.db.profiles[prof].macro[i] and MAM.db.profiles[prof].macro[i].npc then
					for npc, libID in pairs(MAM.db.profiles[prof].macro[i].npc) do
						if not MAM.db.global.library[libID] or MAM.db.global.library[libID] == "deleted" then
							MAM.db.profiles[prof].macro[i].npc[npc] = nil
						end
					end
				end
			elseif typeToClear == "zone" then
				if MAM.db.profiles[prof].macro[i] and MAM.db.profiles[prof].macro[i].zone then
					for zone, libID in pairs(MAM.db.profiles[prof].macro[i].zone) do
						local zoneFound = false
						for zonetype in ipairs(MAM:GetZoneType()) do
							for entry, value in pairs(MAM:FetchZones(zoneType)) do
								if value == zone then
									zoneFound = true
								end
							end
						end
						if not zoneFound then
							MAM.db.profiles[prof].macro[i].zone[zone] = nil
						end
					end
				end
			elseif typeToClear == "npc" then
				if MAM.db.profiles[prof].macro[i] and MAM.db.profiles[prof].macro[i].npc then
					for npc, libID in pairs(MAM.db.profiles[prof].macro[i].npc) do
						local bossTable = MAM:GetBossTable()
						local npcFound = false
						for zone in pairs(bossTable) do
							for entry in pairs(bossTable[zone]) do
								if entry == npc then
									npcFound = true
								end
							end
						end
						for zone in pairs(MAM.db.global.custom.npc) do
							for entry in pairs(MAM.db.global.custom.npc[zone]) do
								if entry == npc then
									npcFound = true
								end
							end
						end
						if not npcFound then
							MAM.db.profiles[prof].macro[i].npc[npc] = nil
						end
					end
				end
			end
		end
	end
end

-- icon choice
function MAM:IconPopUp(currentTexture, macro)
	-- initialize icon table
	MAM.icons = MAM.icons or {}
	
	-- sets current automacro and current texture
	MAM.currentMacro = macro
	MAM.currentTexture = currentTexture
	
	-- gets current icon index
	MAM.currentIconIndex = nil
	for i, v in ipairs(MAM.icons) do
		if v == MAM.currentTexture then
			MAM.currentIconIndex = i
			break
		end
	end
	
	-- create icon interface on first instance
	if not MAM.iconInterfaceMade then
		 MAM:IconInterfaceCreator()
	end

	myAutoMacro_iconFrame:Show()
	myAutoMacro_scrollIconFrame:Show()
end

-- icon scroll frame update
function MAM:ScrollBarIconUpdate()
	local iconOffset = FauxScrollFrame_GetOffset(myAutoMacro_scrollIconFrame)
	
	-- display
	for btn = 1, MAM.ICON_MAX_BUTTON do
		local iconIndex = btn + iconOffset * MAM.NUMBER_ICON_PER_RAW
		if iconIndex <= #MAM.icons then
			MAM.iconBtn[btn].texturePath = MAM.icons[iconIndex]
			MAM.iconBtn[btn].tex:SetTexture(MAM.MACRO_ICON_PATH .. MAM.icons[iconIndex])
			if iconIndex == MAM.currentIconIndex then
				MAM.iconBtn[btn].selected:Show()
			else
				MAM.iconBtn[btn].selected:Hide()
			end
			MAM.iconBtn[btn]:Show()
		else
			MAM.iconBtn[btn].texturePath = ""
			MAM.iconBtn[btn].tex:SetTexture("")
			MAM.iconBtn[btn]:Hide()
		end
	end
	
	local maxLineToShow = math.floor(#MAM.icons / MAM.NUMBER_ICON_PER_RAW) + 1
	if maxLineToShow <= MAM.ICON_MAX_BUTTON/MAM.NUMBER_ICON_PER_RAW then
		maxLineToShow = MAM.ICON_MAX_BUTTON/MAM.NUMBER_ICON_PER_RAW + 1
	end
	
	FauxScrollFrame_Update(myAutoMacro_scrollIconFrame, maxLineToShow, MAM.ICON_MAX_BUTTON/MAM.NUMBER_ICON_PER_RAW, MAM.ICON_RAW_HEIGHT)
end

-- icon choice interface
function MAM:IconInterfaceCreator()
	-- icon frame
	local frame = myAutoMacro_iconFrame
	local width, height = 415, 280
	frame:SetWidth(width)
	frame:SetHeight(height)
	frame:SetPoint("Center", UIParent, "Center")
	frame:SetBackdrop({	bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background", edgeFile = "Interface\\Buttons\\WHITE8X8", edgeSize = 2})
	frame:SetBackdropBorderColor(0, 0.9, 0.89)
	frame:EnableMouse(true)
	frame:SetMovable(true)
	
	frame:SetScript("OnMouseDown", function()
		frame:StartMoving()
	end)
	frame:SetScript("OnMouseUp", function()
		frame:StopMovingOrSizing()
	end)
	frame:Hide()
	
	-- create title
	local title = CreateFrame("Frame", nil, frame)
	title:SetHeight(25)
	title:SetPoint("TOP", frame, "TOP", 0, -10)
	title:SetBackdrop({	bgFile = "", edgeFile = "Interface\\Buttons\\WHITE8X8", edgeSize = 0.75})
	title:SetBackdropBorderColor(0, 0.9, 0.89)
	title.txt = title:CreateFontString()
	title.txt:SetPoint("CENTER", title, "CENTER")
	title.txt:SetFont("Fonts\\FRIZQT__.TTF", 12)
	title.txt:SetShadowColor(0, 0, 0, 1)
	title.txt:SetShadowOffset(1, -1)
	title.txt:SetTextColor(0, 0.9, 0.89)
	title.txt:SetText(L["Click on an icon to select it"])
	title:SetWidth(title.txt:GetStringWidth() * 1.1)
	
	-- scroll frame
	local scroll = myAutoMacro_scrollIconFrame
	scroll:SetPoint("TOPLEFT", frame, "TOPLEFT", 2, -50)
	scroll:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", -45, 60)
	
	-- close button
	local close = CreateFrame("Button", nil, frame, "OptionsButtonTemplate")
	close:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", -20, 10)
	close:SetText(L["Close"])
	close:SetScript("OnClick", function()
		frame:Hide()
		scroll:Hide()
		InterfaceOptionsFrame_OpenToCategory("myAutoMacro")
	end)
	
	-- icon buttons
	local btn = {}
	local btnSize = 30
	local space = 5 -- space between icons
	local border = 20
	local ofx, ofy = 20, 0
	for i = 1, MAM.ICON_MAX_BUTTON do
		btn[i] = CreateFrame("Button", nil, scroll)
		btn[i]:SetWidth(btnSize)
		btn[i]:SetHeight(btnSize)
		btn[i]:SetPoint("TOPLEFT", scroll, "TOPLEFT", ofx, ofy)
		ofx = ofx + btnSize + space
		if ofx > width - (border * 2 + 25) then
			ofx = 20
			ofy = ofy - btnSize - space
		end
		
		btn[i].tex = scroll:CreateTexture()
		btn[i].tex:SetTexCoord(0.08, 0.92, 0.08, 0.92)
		btn[i].tex:SetAllPoints(btn[i])
		
		btn[i].highlight = btn[i]:CreateTexture()
		btn[i].highlight:SetAllPoints(btn[i])
		btn[i].highlight:SetTexture("Interface\\PaperDollInfoFrame\\UI-Character-Tab-Highlight")
		btn[i].highlight:SetTexCoord(0, 1, 0.23, 0.77)
		btn[i].highlight:SetBlendMode("ADD")
		btn[i].highlight:Hide()
		
		btn[i].selected = btn[i]:CreateTexture()
		btn[i].selected:SetAllPoints(btn[i])
		btn[i].selected:SetTexture("Interface\\Buttons\\OldButtonHilight-Square")
		btn[i].selected:SetTexCoord(0.08, 0.92, 0.08, 0.92)
		btn[i].selected:SetBlendMode("ADD")
		btn[i].selected:Hide()
		
		btn[i]:SetScript("OnEnter", function()
			btn[i].highlight:Show()
		end)
		btn[i]:SetScript("OnLeave", function()
			btn[i].highlight:Hide()
		end)
		btn[i]:SetScript("OnMouseDown", function()
			btn[i].selected:Show()
		end)
		btn[i]:SetScript("OnClick", function()
			MAM.db.global.library[MAM.currentMacro].texture = btn[i].texturePath
			for i = 1, MAM.db.profile.general.macroNumber do
				if MAM.db.profile.currentMacros[i] == MAM.currentMacro then
					MAM:UpdateIgMacro(i, MAM.db.global.library[MAM.currentMacro].name, "icon", MAM.db.global.library[MAM.currentMacro].texture)
				end
			end
			frame:Hide()
			InterfaceOptionsFrame_OpenToCategory("myAutoMacro")
		end)		
	end
	MAM.iconBtn = btn
	
	-- search object creation using AIS lib; spell category is obtained using an altered function of blizz macro UI	
	local spellIcons = MAM:GetSpellMacroIcons()
	local options = {
		sections = {
			SpellIcons = {
				count = #spellIcons,
				GetIconInfo = function(index)
					return index, "Spell", spellIcons[index]
				end,
			},
		},
		sectionOrder = {
			"DynamicIcon",
			"SpellIcons",
			"MacroIcons",
			"ItemIcons",
		},
		keywordAddonName = "myAutoMacro_KeywordData",
	}
	local searchObj = AIS:CreateSearch(options)
	searchObj:SetScript("OnSearchStarted", function()
		wipe(MAM.icons)
	end)
	searchObj:SetScript("OnSearchResultAdded", function(search, texture)
		table.insert(MAM.icons, texture)
	end)
	searchObj:SetScript("OnSearchComplete", function()
		-- gets current icon index
		MAM.currentIconIndex = nil
		for i, v in ipairs(MAM.icons) do
			if v == MAM.currentTexture then
				MAM.currentIconIndex = i
				break
			end
		end
		
		-- update scroll frame
		MAM:ScrollBarIconUpdate()
	end)
	searchObj:SetSearchParameter(nil)
	MAM.iconSearchObj = searchObj
	
	-- input box and accompanying stuff
	local searchBox = CreateFrame("EditBox", nil, frame, "InputBoxTemplate")
	local eraseBtn = CreateFrame("Button", nil, frame, "UIPanelCloseButton")
	local message = frame:CreateFontString()

	searchBox:SetWidth(215)
	searchBox:SetHeight(25)
	searchBox:SetPoint("BOTTOMLEFT", frame, "BOTTOMLEFT", 25, 8)
	searchBox:SetAutoFocus(false)
	searchBox:SetScript("OnTextChanged", function(editBox, input)
		if input then
			searchObj:SetSearchParameter(editBox:GetText())
			if editBox:GetText() ~= "" then
				eraseBtn:Show()
			end
		end
	end)
	searchBox:SetScript("OnEscapePressed", function(editBox)
		editBox:ClearFocus()
	end)
	
	eraseBtn:SetWidth(25)
	searchBox:SetHeight(25)
	eraseBtn:SetPoint("LEFT", searchBox, "RIGHT", -2, 0)
	eraseBtn:SetScript("OnClick", function()
		searchBox:SetText("")
		searchObj:SetSearchParameter(searchBox:GetText())
		eraseBtn:Hide()
	end)
	eraseBtn:Hide()
	
	message:SetPoint("BOTTOMLEFT", searchBox, "TOPLEFT", -4, 1)
	message:SetFont("Fonts\\FRIZQT__.TTF", 11)
	message:SetShadowColor(0, 0, 0, 1)
	message:SetShadowOffset(1, -1)
	message:SetTextColor(0, 0.9, 0.89)
	message:SetText(L["Search icons (English only!):"])
	
	MAM.iconInterfaceMade = true
end

-- fetches player spell icons
function MAM:GetSpellMacroIcons()
	local spellIcons = {}
	local _, _, offset, numSpells = GetSpellTabInfo(GetNumSpellTabs())
	local totalSpells = offset + numSpells
	for i = 1, totalSpells do
		skillType, skillID = GetSpellBookItemInfo(i, "player")
		if skillType ~= "FUTURESPELL" then
			local spellTexture = string.upper(GetSpellBookItemTexture(i, "player"))
			if not string.find(spellTexture, "INTERFACE\\BUTTONS\\") then
				spellTexture = string.gsub(spellTexture, "INTERFACE\\ICONS\\", "")
				table.insert(spellIcons, spellTexture)
			end
			if skillType == "FLYOUT" then
				local _, _, numSlots, isKnown = GetFlyoutInfo(skillID)
				if isKnown and numSlots > 0 then
					for k = 1, numSlots do
						local spellID, isKnown = GetFlyoutSlotInfo(skillID, k)
						if isKnown then
							spellTexture = string.upper(GetSpellTexture(spellID))
							spellTexture = string.gsub(spellTexture, "INTERFACE\\ICONS\\", "")
							table.insert(spellIcons, spellTexture)
						end
					end
				end
			end
		end
	end

	return spellIcons
end

-- summarizes current associations
function MAM:ShowAssociations()
	if not MAM.InterfaceSumMade then
		MAM:InterfaceSumCreator()
	end
	UIDropDownMenu_Initialize(myAutoMacro_SumFrameDropDown, MAM.UpdateDropdown)
	MAM:PopulateSumFrame(myAutoMacro_SumFrameDropDown, 1)
	
	if not MAM.showAssoFromCmd then
		InterfaceOptionsFrame:Hide()
		HideUIPanel(GameMenuFrame)
	end
	myAutoMacro_sumFrame:Show()
	myAutoMacro_scrollSumFrame:Show()
end

-- creates association frame
function MAM:InterfaceSumCreator()
	-- association frame
	local frame = myAutoMacro_sumFrame
	local width, height = 600, 500
	frame:SetWidth(width)
	frame:SetHeight(height)
	frame:SetPoint("TOP", UIParent, "TOP", 0, -100)
	frame:SetBackdrop({	bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background", edgeFile = "Interface\\Buttons\\WHITE8X8", edgeSize = 2})
	frame:SetBackdropBorderColor(0, 0.9, 0.89)
	frame:EnableMouse(true)
	frame:SetMovable(true)

	frame:SetScript("OnMouseDown", function()
		frame:StartMoving()
	end)
	frame:SetScript("OnMouseUp", function()
		frame:StopMovingOrSizing()
	end)
	frame:Hide()
	
	-- title
	local title = CreateFrame("Frame", nil, frame)
	title:SetHeight(25)
	title:SetPoint("TOP", frame, "TOP", 0, -10)
	title:SetBackdrop({	bgFile = "", edgeFile = "Interface\\Buttons\\WHITE8X8", edgeSize = 0.75})
	title:SetBackdropBorderColor(0, 0.9, 0.89)
	title.txt = title:CreateFontString()
	title.txt:SetPoint("CENTER", title, "CENTER")
	title.txt:SetFont("Fonts\\FRIZQT__.TTF", 12)
	title.txt:SetShadowColor(0, 0, 0, 1)
	title.txt:SetShadowOffset(1, -1)
	title.txt:SetTextColor(0, 0.9, 0.89)
	title.txt:SetText(L["myAutoMacro current associations"])
	title:SetWidth(title.txt:GetStringWidth() * 1.1)
	
	-- scroll frame
	local scroll = myAutoMacro_scrollSumFrame
	scroll:SetPoint("TOPLEFT", frame, "TOPLEFT", 2, -110)
	scroll:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", -30, 50)
	
	-- close button
	local close = CreateFrame("Button", nil, frame, "OptionsButtonTemplate")
	close:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", -20, 10)
	close:SetText(L["Close"])
	close:SetScript("OnClick", function()
		frame:Hide()
		scroll:Hide()
		if not MAM.showAssoFromCmd then
			InterfaceOptionsFrame_OpenToCategory("myAutoMacro")
		end
		MAM.showAssoFromCmd = false
	end)
	
	-- lines
	MAM.assoList.lineLeft = MAM.assoList.lineLeft or {}
	MAM.assoList.lineRight = MAM.assoList.lineRight or {}
	MAM.assoList.lineCenter = MAM.assoList.lineCenter or {}
	for i = 1, 17 do
		MAM.assoList.lineLeft[i] = frame:CreateFontString()
		MAM.assoList.lineLeft[i]:SetPoint("TOPLEFT", scroll, "TOPLEFT", 5, - 20 * (i - 1))
		MAM.assoList.lineLeft[i]:SetFont("Fonts\\FRIZQT__.TTF", 12)
		MAM.assoList.lineLeft[i]:SetShadowColor(0, 0, 0, 1)
		MAM.assoList.lineLeft[i]:SetShadowOffset(1, -1)
		MAM.assoList.lineLeft[i]:SetJustifyH("LEFT")
		
		MAM.assoList.lineRight[i] = frame:CreateFontString()
		MAM.assoList.lineRight[i]:SetPoint("TOPRIGHT", scroll, "TOPRIGHT", -2, - 20 * (i - 1))
		MAM.assoList.lineRight[i]:SetFont("Fonts\\FRIZQT__.TTF", 12)
		MAM.assoList.lineRight[i]:SetShadowColor(0, 0, 0, 1)
		MAM.assoList.lineRight[i]:SetShadowOffset(1, -1)
		MAM.assoList.lineRight[i]:SetJustifyH("RIGHT")
		
		
		MAM.assoList.lineCenter[i] = frame:CreateFontString()
		MAM.assoList.lineCenter[i]:SetPoint("TOP", scroll, "TOP", 0, - 20 * (i - 1))
		MAM.assoList.lineCenter[i]:SetFont("Fonts\\FRIZQT__.TTF", 12)
		MAM.assoList.lineCenter[i]:SetShadowColor(0, 0, 0, 1)
		MAM.assoList.lineCenter[i]:SetShadowOffset(1, -1)
		MAM.assoList.lineCenter[i]:SetJustifyH("CENTER")
		MAM.assoList.lineCenter[i]:SetJustifyV("CENTER")	
	end
	
	-- dropdown for automacros
	local dropdown = CreateFrame("Button", "myAutoMacro_SumFrameDropDown", frame, "UIDropDownMenuTemplate")
	dropdown:SetWidth(150)
	dropdown:SetHeight(15)
	dropdown:SetPoint("TOPLEFT", frame, "TOPLEFT", 2, -60)
	
	MAM.InterfaceSumMade = true
end

-- initialize function for dropdown
function MAM:UpdateDropdown(level)
	local info = UIDropDownMenu_CreateInfo()
	for i = 1, MAM.db.profile.general.macroNumber do
		wipe(info)
		info.text = "Automacro" .. i
		info.value = "Automacro" .. i
		info.func = function() MAM:PopulateSumFrame(myAutoMacro_SumFrameDropDown, i) end
		UIDropDownMenu_AddButton(info, level)
	end
end

-- populates the association frame with current association for a given automacro
function MAM:PopulateSumFrame(menu, automacro)
	UIDropDownMenu_SetSelectedID(menu, automacro)
	
	local bosses = {}
	local npcs = {}
	local zones = {}
	local macroName = ""
	MAM.assoList.macros = MAM.assoList.macros or {}
	wipe(MAM.assoList.macros)
	MAM.assoList.associates = MAM.assoList.associates or {}
	wipe(MAM.assoList.associates)
	
	--we scan boss table in addon db, get the name if not already fetched (data redundancy), scan custom npc table
	local npcName = ""
	local bossTable = MAM:GetBossTable()
	for npc, macro in pairs(MAM.db.profile.macro[automacro].npc) do
		local isNPC, isBoss = false, false
		for zone in pairs(bossTable) do
			for entry in pairs(bossTable[zone]) do
				if entry == npc then
					npcName = MAM:GetBossName(zone, npc)
					isBoss = true
				end
			end
		end
		-- we check custom npc now
		if not isBoss then
			for zone in pairs(MAM.db.global.custom.npc) do
				for entry in pairs(MAM.db.global.custom.npc[zone]) do
					if entry == npc then
						npcName = MAM.db.global.custom.npc[zone][entry]
						isNPC = true
					end
				end
			end
		end
		-- gets macro name
		for entry in ipairs(MAM.db.global.library) do
			if entry == macro then
				macroName = MAM.db.global.library[entry].name
			end
		end
		
		-- throws an error if macroName or npcName is empty
		assert(npcName ~="", L["Database is corrupted. Please reset entirely the database or contact your sys admin ;)."])
		assert(macroName ~="", L["Database is corrupted. Please reset entirely the database or contact your sys admin ;)."])
		
		-- inserts data in bosses or npc table, replaces npcName if necessary
		if npcName == "" then npcName = L["Unknown"] end
		if isBoss and not bosses[npcName] then
			bosses[npcName] = macroName
			bosses[1] = true
		elseif isNPC and not npcs[npcName] then
			npcs[npcName] = macroName
			npcs[1] = true
		end
		
		-- resets macroName
		macroName = ""
	end
	
	-- same for zones
	local zoneName = ""
	for zone, macro in pairs(MAM.db.profile.macro[automacro].zone) do
		zoneName = MAM:FetchZoneName(zone)
		
		-- macroName
		for entry in ipairs(MAM.db.global.library) do
			if entry == macro then
				macroName = MAM.db.global.library[entry].name
			end
		end
		
		-- error
		assert(zoneName ~="", L["Database is corrupted. Please reset entirely the database or contact your sys admin ;)."])
		assert(macroName ~="", L["Database is corrupted. Please reset entirely the database or contact your sys admin ;)."])

		-- inserts data
		if zoneName == "" then zoneName = L["Unknown"] end
		if macroName ~= "" then
			zones[zoneName] = macroName
			zones[1] = true
		end
		
		-- resets macroName
		macroName = ""
	end
	
	-- inserts entries of the 3 tables into 2 tables containing the text displayed association frame
	if #bosses > 0 then
		table.insert(MAM.assoList.associates, L["Boss associations:"])
		table.insert(MAM.assoList.macros, "")
		table.insert(MAM.assoList.associates, MAM.NORMAL_COLOR .. L["Boss"])
		table.insert(MAM.assoList.macros, MAM.NORMAL_COLOR .. L["Macro"])
		for npc in pairs(bosses) do
			if npc ~= 1 then
				table.insert(MAM.assoList.associates, format(MAM.HIGHLIGHT_COLOR .. npc))
				table.insert(MAM.assoList.macros, format(MAM.HIGHLIGHT_COLOR .. bosses[npc]))
			end
		end
		table.insert(MAM.assoList.associates, "")
		table.insert(MAM.assoList.macros, "")
	end
	if #npcs > 0 then
		table.insert(MAM.assoList.associates, L["NPC associations:"])
		table.insert(MAM.assoList.macros, "")
		table.insert(MAM.assoList.associates, MAM.NORMAL_COLOR .. L["NPC"])
		table.insert(MAM.assoList.macros, MAM.NORMAL_COLOR .. L["Macro"])
		for npc in pairs(npcs) do
			if npc ~= 1 then
				table.insert(MAM.assoList.associates, format(MAM.HIGHLIGHT_COLOR .. npc))
				table.insert(MAM.assoList.macros, format(MAM.HIGHLIGHT_COLOR .. npcs[npc]))
			end
		end
		table.insert(MAM.assoList.associates, "")
		table.insert(MAM.assoList.macros, "")
	end
	if #zones > 0 then
		table.insert(MAM.assoList.associates, L["Zone associations:"])
		table.insert(MAM.assoList.macros, "")
		table.insert(MAM.assoList.associates, MAM.NORMAL_COLOR .. L["Zone"])
		table.insert(MAM.assoList.macros, MAM.NORMAL_COLOR .. L["Macro"])
		for zone in pairs(zones) do
			if zone ~= 1 then
				table.insert(MAM.assoList.associates, format(MAM.HIGHLIGHT_COLOR .. zone))
				table.insert(MAM.assoList.macros, format(MAM.HIGHLIGHT_COLOR .. zones[zone]))
			end
		end
		table.insert(MAM.assoList.associates, "")
		table.insert(MAM.assoList.macros, "")
	end
	
	if #MAM.assoList.associates == 0 then
		table.insert(MAM.assoList.associates, L["This automacro has no association."])
		table.insert(MAM.assoList.macros, "")
	end
	
	MAM:ScrollBarSumUpdate()
end

-- update association scrollframe
function MAM:ScrollBarSumUpdate()
	local lineOffset
	
	-- display
	for i = 1, 17 do
		lineOffset = i + FauxScrollFrame_GetOffset(myAutoMacro_scrollSumFrame)
		if lineOffset <= #MAM.assoList.associates then
			MAM.assoList.lineLeft[i]:SetText(MAM.assoList.associates[lineOffset])
			if string.find(MAM.assoList.associates[lineOffset], ":") then
				MAM.assoList.lineLeft[i]:SetFont("Fonts\\FRIZQT__.TTF", 14)
			else
				MAM.assoList.lineLeft[i]:SetFont("Fonts\\FRIZQT__.TTF", 12)
			end
			MAM.assoList.lineRight[i]:SetText(MAM.assoList.macros[lineOffset])
			MAM.assoList.lineCenter[i]:SetText(MAM.NORMAL_COLOR .. format("<\226\128\148>"))
			
			MAM.assoList.lineLeft[i]:Show()
			MAM.assoList.lineRight[i]:Show()
			if MAM.assoList.macros[lineOffset] and MAM.assoList.associates[lineOffset] and (not string.find(MAM.assoList.macros[lineOffset], "^.+") or string.find(MAM.assoList.associates[lineOffset], L["Boss"]) or string.find(MAM.assoList.associates[lineOffset], L["NPC"]) or string.find(MAM.assoList.associates[lineOffset], L["Zone"])) then
				MAM.assoList.lineCenter[i]:Hide()
			else
				MAM.assoList.lineCenter[i]:Show()
			end
		else
			MAM.assoList.lineLeft[i]:Hide()
			MAM.assoList.lineRight[i]:Hide()
			MAM.assoList.lineCenter[i]:Hide()
		end
	end
	
	FauxScrollFrame_Update(myAutoMacro_scrollSumFrame, #MAM.assoList.associates, 17, 20)
end
